/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

/**
 * An extension of PreviousExpression for recording the location of a control on a form.
 * Adds RIGHT@<exp>, BOTTOM@<exp>, CENTER@<exp> and CENTER formats for showing alignment.
 */
public class LocationExpression extends PreviousExpression
{
  public static final int ALIGN_LEFT=1;
  public static final int ALIGN_TOP=1;
  public static final int ALIGN_CENTER=2;
  public static final int ALIGN_CENTER_AT=3;
  public static final int ALIGN_BOTTOM=4;
  public static final int ALIGN_RIGHT=5;

	/** The alignment of this expression - one of the ALIGN_* constants */
	int align=ALIGN_TOP;

  /**
   * Creates a new LocationExpression.
   */
  public LocationExpression()
  {
	}

  public LocationExpression(LocationExpression exp)
  {
		super(exp);
	}

  /**
   * Creates a new LocationExpression.
   * @param obj the previous object for PREV* constants
   */
	public LocationExpression(FormObject obj)
	{
		super(obj);
	}

  public int getAlignment()
  {
		return align;
	}

  public boolean isRightAligned()
  {
		return align==ALIGN_RIGHT;
	}

	public boolean isBottomAligned()
	{
		return align==ALIGN_BOTTOM;
	}

	public boolean isCenterAligned()
	{
		return align==ALIGN_CENTER;
	}

  /**
   * Gets this expression as a string.
   * @return the expression
   */
  public String getExpression()
  {
		if (align==ALIGN_LEFT||align==ALIGN_TOP)
		  return super.getExpression();
		if (align==ALIGN_CENTER)
		  return Keywords.get(this,ALIGN_CENTER);
		if (align==ALIGN_BOTTOM||align==ALIGN_RIGHT)
		  return Keywords.get(this,align)+super.getExpression();

		return super.getExpression(); // shouldn't happen
	}

  public void setValue(int i)
  {
		align=ALIGN_TOP; // or ALIGN_RIGHT
		super.setValue(i);
	}

  /**
   * Sets the expression string.
   * @param s the string to set
   */
  public void setExpression(String s)
  {
		if (Keywords.get(this,s)==ALIGN_CENTER)
		{
		  align=ALIGN_CENTER;
		  super.setExpression(null);
		  return;
		}

		align=ALIGN_TOP; // or ALIGN_RIGHT
		int preindex=s.indexOf('@');
		if (preindex!=-1)
		{
      int key=Keywords.get(this,s.substring(0,preindex+1));
      if (key!=-1)
        align=key;
			s=s.substring(preindex+1);
		}
		super.setExpression(s);
	}
}
