/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class Properties extends Panel implements SelectionListener
{
	public static final int TEXTFIELD=Property.TEXTFIELD;
	public static final int LIST=Property.TF_LIST;
	public static final int MULTILINE=Property.TF_MULTILINE;

	DynamicGridLayout layout;
	Hashtable oldProperties;
	Hashtable properties=new Hashtable();
	FormObject obj;
	Screen screen;
	java.awt.Label emptyLabel;
	java.awt.Label objType;
	boolean sameClass=false;
	java.awt.TextField focusTextField;
	TextEditor textEditor;

	public Properties(Screen screen)
	{
		this.screen=screen;
		setLayout(layout=new DynamicGridLayout(20,2,5,2));
		layout.setStyle(layout.LABEL_FILL);
	}

  public void formSelected(Form form) {}

  public void refresh()
  {
    controlSelected(obj,true);
  }

  public void controlSelected(FormObject obj)
  {
    controlSelected(obj,false);
  }

  public void controlSelected(FormObject obj,boolean refresh)
	{
		if (obj==null)
		{
			setVisible(false);
			return;
		}

		if (this.obj==obj&&!refresh)
		  return;
		if (this.obj!=null)
		  this.obj.propertiesClosed();

    if (sameClass=(this.obj!=null&&obj.getClass()==this.obj.getClass()))
    {
			this.obj=obj;
			oldProperties=properties;
			obj.getProperties(this);
		}
		else
		{
			setVisible(false);
			removeAll();
			if (emptyLabel==null)
			{
				emptyLabel=new java.awt.Label();
				objType=new java.awt.Label();
			}
			add(emptyLabel);
			objType.setText(obj.getDisplayName());
			add(objType);

			this.obj=obj;
			oldProperties=properties;
			focusTextField=null;
			properties=new Hashtable();
			obj.getProperties(this);
			setVisible(true);
			getParent().validate();
		}
	}

  public void addProperty(String name,String value)
  {
		addProperty(name,value,TEXTFIELD,false);
	}

  public void addProperty(String name,String value,int type)
  {
		addProperty(name,value,type,false);
	}

  public void addProperty(String name,String value,boolean main)
  {
		addProperty(name,value,TEXTFIELD,main);
	}

	public void addProperty(String name,String value,int type,boolean main)
	{
		Property prop=(Property)oldProperties.get(name);
		if (prop==null)
		  prop=new Property(name,value,type);
		else
		{
		  prop.setValue(value);
		  prop.setType(type);
		}
		if (!sameClass)
  		prop.addTo(this);
    properties.put(name,prop);
    if (main)
      focusTextField=prop.valueTF;
	}

  public void addProperty(String name,String[] values,int index)
  {
    addProperty(name,values,index,false);
  }

  public void addProperty(String name,String[] values,int index,boolean alwaysDrop)
  {
		Property prop=(Property)oldProperties.get(name);
		if (prop==null)
    {
      if (alwaysDrop)
        prop=new Property(name,values,index,Property.DROP_CHOICE);
      else
  		  prop=new Property(name,values,index);
    }
		else
  		prop.setValues(values,index);
    if (!sameClass)
      prop.addTo(this);
    properties.put(name,prop);
	}

  public void setProperty(String name,String value)
  {
		Property prop=(Property)properties.get(name);
		if (prop!=null)
		  prop.setValue(value);

	}

  public void setProperty(String name,int index)
  {
		Property prop=(Property)properties.get(name);
		if (prop!=null)
		  prop.setValue(index);
	}

  public void editorKeyPressed(KeyEvent e)
  {
    char ch=e.getKeyChar();
    if (ch!=e.CHAR_UNDEFINED&&ch>32&&focusTextField!=null)
    {
      String s=""+ch;
      focusTextField.setText(""+ch);
      focusTextField.setCaretPosition(1);
      new Focuser(focusTextField).start();
    }
  }

	class Property implements TextListener,MouseListener,ItemListener,ActionListener,FocusListener
	{
		private static final int CLICK_CHOICE=0;
		private static final int DROP_CHOICE=1;
		private static final int TEXTFIELD=2;
		private static final int TF_LIST=3;
		private static final int TF_MULTILINE=4;

		int type=-1;
		int index=-1;
		boolean valid=true;

		String name;
		java.awt.Label nameLabel;
		String valueTFLast;
		TextField valueTF;
		Choice valueChoice;
		PopupMenu popup;
		String[] values;
		boolean textSelected=false;

		public Property(String name,String value,int type)
		{
		  this.type=type;
			this.name=name;
			nameLabel=new java.awt.Label(name,java.awt.Label.RIGHT);
			valueTF=new TextField(valueTFLast=(value!=null?value:""));
			valueTF.addActionListener(this);
			valueTF.addTextListener(this);
			valueTF.addMouseListener(this);
			valueTF.addFocusListener(this);
		}

    public Property(String name,String[] values,int index)
    {
      this(name,values,index,values.length<=3?CLICK_CHOICE:DROP_CHOICE);
    }

    public Property(String name,String[] values,int index,int type)
    {
			this.name=name;
			this.values=values;
			nameLabel=new java.awt.Label(name,java.awt.Label.RIGHT);
      this.type=type;
      if (type==CLICK_CHOICE)
      {
				this.index=index;
				valueTF=new TextField(values[index]);
				valueTF.addMouseListener(this);
	      valueTF.setEditable(false);
			}
			else
			{
				valueChoice=new Choice();
				for(int i=0;i<values.length;i++)
				  valueChoice.add(values[i]);
				valueChoice.select(index);
				valueChoice.addItemListener(this);
			}
		}

    public void addTo(Container c)
    {
		  c.add(nameLabel);
		  if (type==DROP_CHOICE)
		    c.add(valueChoice);
		  else
		    c.add(valueTF);
		}

    public void focusGained(FocusEvent e)
    {
			//valueTF.selectAll();
			valueTF.setCaretPosition(valueTF.getText().length());
		}

		public void focusLost(FocusEvent e)
		{
			valueTF.select(0,0);
		}

    public void actionPerformed(ActionEvent e)
    {
			if (!valid)
			  try
			  {
  			  obj.setProperty(name,valueTFLast,-2);
  			  valueTF.setForeground(Color.black);
  			  screen.repaint();
				}
				catch(IllegalArgumentException ex)
				{
					// ignore
				}
		}

	  public void textValueChanged(TextEvent e)
	  {
			valid=true;
			String s=valueTF.getText();
			if (!s.equals(valueTFLast))
			{
				try
				{
					//System.out.println("textValueChanged("+s+")");
					valueTFLast=s;
  				obj.setProperty(name,s,-1);
  				valueTF.setForeground(Color.black);
  				screen.repaint();
  				obj.getProject().setDirty(true);
				}
				catch(IllegalArgumentException ex)
				{
					valueTF.setForeground(Color.red);
					valid=false;
				}
			}
		}

    public void setValues(String[] s,int index)
    {
			if (s!=values)
			{
			  values=s;
			  if (type==DROP_CHOICE)
			  {
					valueChoice.removeAll();
					for(int i=0;i<values.length;i++)
						valueChoice.add(values[i]);
			  }
		  }
		  setValue(index);
		}

		public void setValue(String s)
		{
			valueTF.setForeground(Color.black);
			if (type>=TEXTFIELD&&(valueTFLast==null||!valueTFLast.equals(s)))
			{
        if (s==null)
          s="";
				//System.out.println("setValue("+s+")");
				valueTFLast=s;
				valueTF.setText(s);
			}
		}

    public void setType(int type)
    {
			if (this.type>=TEXTFIELD&&type>=TEXTFIELD)
			  this.type=type;
		}

    public void setValue(int index)
    {
			if (type==DROP_CHOICE)
			  valueChoice.select(index);
			else
			if (type==CLICK_CHOICE)
			{
				this.index=index;
			  valueTF.setText(values[index]);
			}
		}

    public void itemStateChanged(ItemEvent e)
    {
			obj.getProject().setDirty(true);
			int i=valueChoice.getSelectedIndex();
			obj.setProperty(name,values[i],i);
			screen.repaint();
		}

		/**
		 * Invoked when the mouse has been clicked on a component.
		 */
		public void mouseClicked(MouseEvent e)
		{
			//System.out.println("mouseClicked("+e.getClickCount()+")");
			if (type>TEXTFIELD)
			{
				if (e.getClickCount()>=2)
				{
					if (textEditor==null)
						textEditor=new TextEditor();
					textEditor.setListMode(type==TF_LIST);
					textEditor.setTextField(name+" Editor",valueTF);
					textEditor.setVisible(true);
					//e.consume();
				}
			}
			else
			if (type==TEXTFIELD)
			{
				if (textSelected)
				{
					int len=valueTF.getText().length();
				  valueTF.setSelectionStart(len);
				  valueTF.setSelectionEnd(len);
				}
				else
				{
				  valueTF.selectAll();
				}
			}

			//if (e.getModifiers()&&e.BUTTON_3_CLICKED
			//e.consume();
	  }

		/**
		 * Invoked when a mouse button has been pressed on a component.
		 */
		public void mousePressed(MouseEvent e)
		{
			if (type==TEXTFIELD)
  			textSelected=valueTF.getSelectionStart()!=valueTF.getSelectionEnd();
		}

		/**
		 * Invoked when a mouse button has been released on a component.
		 */
		public void mouseReleased(MouseEvent e)
		{
			/*
			if (e.isPopupTrigger())
			{
				popup.show(e.getComponent(),e.getX(),e.getY());
				e.consume();
			}
			else
			*/
			if (type==CLICK_CHOICE)
			{
				obj.getProject().setDirty(true);
				index=(index+1)%values.length;
				valueTF.setText(values[index]);
				obj.setProperty(name,values[index],index);
				screen.repaint();
			}
		}

		/**
		 * Invoked when the mouse enters a component.
		 */
		public void mouseEntered(MouseEvent e) {}

		/**
		 * Invoked when the mouse exits a component.
		 */
		public void mouseExited(MouseEvent e) {}

  }
}