/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.io.*;

public abstract class Resource
{
  protected IDExpression id=new IDExpression("AUTOID");
  private ChangeListeners listeners;

  protected String readNormalString(StreamTokenizer st) throws IOException
  {
    st.lowerCaseMode(false);
    st.nextToken();
    st.lowerCaseMode(true);
    return st.sval;
  }

  protected String readQuotedString(StreamTokenizer st) throws IOException
  {
    st.ordinaryChar('"');
    st.ordinaryChar('\\');
    st.lowerCaseMode(false);
    st.nextToken();
    String s=null;
    StringBuffer sb=null;
    if (st.ttype!='"')
      st.pushBack();
    else
    {
      st.ordinaryChar(' ');
      st.ordinaryChar('\t');
      st.wordChars(' ',' ');
      st.wordChars('\t','\t');
      st.nextToken();
      s="";
      while (st.ttype!='"')
      {
        if (st.ttype=='\\')
        {
          if (sb==null)
            sb=new StringBuffer(s);
          st.nextToken();
          if (st.ttype=='"'||st.ttype=='\\')
            sb.append((char)st.ttype);
          else
          if (st.ttype==st.TT_WORD)
            sb.append('\\').append(st.sval);
        }
        else
        if (st.ttype==st.TT_WORD)
        {
          if (sb!=null)
            sb.append(st.sval);
          else
            s=st.sval;
        }
        st.nextToken();
      }
    }
    st.whitespaceChars(' ',' ');
    st.whitespaceChars('\t','\t');
    st.lowerCaseMode(true);
    st.wordChars('"','"');
    st.wordChars('\\','\\');
    if (sb==null)
      return s;
    else
      return sb.toString();
  }

  protected String parseID(StreamTokenizer st) throws IOException
  {
    st.lowerCaseMode(false);
    st.nextToken();
    if (st.sval.equalsIgnoreCase("ID"))
      st.nextToken();
    st.lowerCaseMode(true);
    return st.sval;
  }

  protected void readID(StreamTokenizer st) throws IOException
  {
    id.setExpression(parseID(st));
  }

  public abstract void parse(StreamTokenizer st) throws IOException;

  public abstract void write(PrintStream ps);

  public int getIDValue()
  {
    return id.getValue();
  }

  public String getID()
  {
    return id.toString();
  }

  public String toString()
  {
    return getClass()+"-"+id+"-"+getIDValue();
  }

  public void addChangeListener(ChangeListener listener)
  {
    if (listeners==null)
      listeners=new ChangeListeners();
    listeners.add(listener);
  }

  public void removeChangeListener(ChangeListener listener)
  {
    if (listeners!=null)
      listeners.remove(listener);
  }

  protected void fireObjectChanged(int id)
  {
    fireObjectChanged(id,null);
  }

  protected void fireObjectChanged(int id,Object data)
  {
    if (listeners!=null)
      listeners.fireObjectChanged(this,id,data);
  }
}