/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit.control;

import pilrcedit.*;
import java.awt.Graphics;
import java.awt.Color;
import java.io.*;

/**
 * Class representing a standard Palm button
 */
public class Button extends FormObject
{
  public static String[] FRAME_TYPES;
  public static final int BOLDFRAME=0;
  public static final int NOFRAME=1;
  public static final int FRAME=2;

  public static final int PROP_FRAME=3;

	private boolean boldFrame;
	private boolean noFrame;

  protected boolean parseOption(StreamTokenizer st) throws IOException
  {
		if (super.parseOption(st))
		  return true;
    int key=Keywords.get(this,st.sval);
    switch(key)
    {
      case BOLDFRAME:
        boldFrame=true;
        break;
      case NOFRAME:
        noFrame=true;
        break;
      case FRAME:
        //ignore
        break;
      default:
        return false;
    }
		return true;
  }

	protected void writeOptions(StringBuffer sb)
	{
		super.writeOptions(sb);
		if (boldFrame)
		  sb.append(' ').append(Keywords.get(this,BOLDFRAME));
		else
		if (noFrame)
		  sb.append(' ').append(Keywords.get(this,NOFRAME));
	}

  public void getProperties(Properties prop)
  {
		super.getProperties(prop);
		prop.addProperty(Keywords.get(this,PROP_FRAME),FRAME_TYPES,noFrame?0:boldFrame?2:1);
	}

  public boolean setProperty(String name,String value,int index)
  {
		if (Keywords.get(this,name)==PROP_FRAME)
	  {
			noFrame=index==0;
			boldFrame=index==2;
	  }
		else
		  return super.setProperty(name,value,index);
		evaluatePosition();
		return true;
	}

  public int autoWidth()
  {
		return super.autoWidth()+12;
	}

  public void paint(Graphics g)
  {
    //System.out.println(label+".paint("+boldFrame+":"+noFrame+")");
		int x1=x;
		int y1=y;
		int x2=x1+width-1;
		int y2=y1+height-1;
		g.setColor(usable?Color.black:Color.lightGray);
		String displayLabel=TextTools.unescape(label);
		Font font=getFont();
		font.drawString(g,displayLabel,x1+(width-font.stringWidth(displayLabel)+1)/2,
		  y1+(height-font.getHeight())/2);
		if (boldFrame)
		{
			g.drawLine(x1+3,y1-2,x2-3,y1-2);
			g.drawLine(x1+1,y1-1,x2-1,y1-1);
			g.drawLine(x1,y1,x1+3,y1);
			g.drawLine(x2-3,y1,x2,y1);
			g.drawLine(x1,y1+1,x1+1,y1+1);
			g.drawLine(x2-1,y1+1,x2,y1+1);
			g.drawLine(x1,y1+2,x1,y1+3);
			g.drawLine(x2,y1+2,x2,y1+3);
			g.drawLine(x1-1,y1+1,x1-1,y2-1);
			g.drawLine(x2+1,y1+1,x2+1,y2-1);
			g.drawLine(x1-2,y1+3,x1-2,y2-3);
			g.drawLine(x2+2,y1+3,x2+2,y2-3);
			g.drawLine(x1,y2-2,x1,y2-3);
			g.drawLine(x2,y2-2,x2,y2-3);
			g.drawLine(x2-1,y2-1,x2,y2-1);
			g.drawLine(x1,y2-1,x1+1,y2-1);
			g.drawLine(x2-3,y2,x2,y2);
			g.drawLine(x1,y2,x1+3,y2);
			g.drawLine(x1+1,y2+1,x2-1,y2+1);
			g.drawLine(x1+3,y2+2,x2-3,y2+2);
		}
		else
		if (!noFrame)
		{
			g.drawLine(x1,y1,x1+1,y1);
			g.drawLine(x1+2,y1-1,x2-2,y1-1);
			g.drawLine(x2-1,y1,x2,y1);
			g.drawLine(x1,y1+1,x1,y1+1);
			g.drawLine(x2,y1+1,x2,y1+1);
			g.drawLine(x1-1,y1+2,x1-1,y2-2);
			g.drawLine(x2+1,y1+2,x2+1,y2-2);
			g.drawLine(x1,y2-1,x1,y2-1);
			g.drawLine(x2,y2-1,x2,y2-1);
			g.drawLine(x1,y2,x1+1,y2);
			g.drawLine(x2-1,y2,x2,y2);
			g.drawLine(x1+2,y2+1,x2-2,y2+1);
		}
		g.setColor(Color.black);
  }
}